"use strict";
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
function validateBoolOption(name, value, defaultValue) {
    if (typeof value === 'undefined') {
        value = defaultValue;
    }
    if (typeof value !== 'boolean') {
        throw new Error(`@justia/babel-preset: '${name}' option must be a boolean.`);
    }
    return value;
}
module.exports = (0, helper_plugin_utils_1.declare)((api, options) => {
    api.assertVersion('^7.0.0');
    const env = process.env.BABEL_ENV || process.env.NODE_ENV || 'development';
    const isEnvDevelopment = env === 'development';
    const isEnvProduction = env === 'production';
    const isEnvTest = env === 'test';
    const { targets = {}, useBuiltIns = 'entry', corejs = 3, include = [], exclude = [], modules = 'auto', typescript = false } = options;
    if (typeof modules !== 'boolean' && modules !== 'auto') {
        throw new TypeError('`@justia/babel-preset` only accepts `true`, `false`, or `"auto"` as the value of the "modules" option');
    }
    const toExclude = [
        ...new Set([
            'transform-typeof-symbol',
            ...exclude
        ])
    ];
    const isTypeScriptEnabled = validateBoolOption('typescript', typescript, false);
    const debug = validateBoolOption('debug', options.debug, false);
    const development = validateBoolOption('development', options.development, api.cache.using(() => isEnvDevelopment));
    return {
        presets: [
            isEnvTest && [
                require('@babel/preset-env'),
                {
                    targets: {
                        node: 'current'
                    }
                }
            ],
            (isEnvProduction || isEnvDevelopment) && [
                require('@babel/preset-env'),
                {
                    modules: modules === false ? false : 'auto',
                    debug,
                    include,
                    exclude: toExclude,
                    useBuiltIns,
                    corejs,
                    targets
                }
            ],
            [
                require('@babel/preset-react'),
                {
                    development: development || isEnvTest,
                    useBuiltIns: true
                }
            ],
            isTypeScriptEnabled && [require('@babel/preset-typescript')]
        ].filter(Boolean),
        plugins: [
            isTypeScriptEnabled && [require('@babel/plugin-proposal-decorators'), { version: 'legacy' }],
            require('@babel/plugin-transform-numeric-separator')
        ].filter(Boolean)
    };
});
